@@ -25,9 +25,12 @@ from typing import List, Optional, Tuple
 import nibabel as nib
 import numpy as np
 
+LOGGER = logging.getLogger(__name__)
+
 
 def load_me_data(pattern: Path, TEs: Optional[Tuple[float]]) -> Tuple[List[Tuple[nib.Nifti1Image, float]], list]:
-    """Load all echoes and their TEs.
+    """
+    Load all echoes and their TEs.
     Return a list of tuples like:
     [(echo1, TE1), (echo2, TE2), ..., (echoN, TEN)]
     Here, echoN is a numpy array of loaded data.
@@ -47,17 +50,17 @@ def load_me_data(pattern: Path, TEs: Optional[Tuple[float]]) -> Tuple[List[Tuple
     LOGGER.info(f'Multi-Echo times: {TEs}')
     LOGGER.info(f'Loading ME-files: {[str(datafile) for datafile in datafiles]}')
 
-    return [(nib.load(str(datafile)), TEs[n]) for n, datafile in enumerate(datafiles)], datafiles
+    return [(nib.load(datafile), TEs[n]) for n, datafile in enumerate(datafiles)], list(datafiles)
 
 
 def paid_weights(echoes: List[Tuple[nib.Nifti1Image, float]], n_vols: int) -> np.array:
-    """Compute PAID weights from echoes described as a list of tuples,
-    as loaded by load_me_data.
+    """
+    Compute PAID weights from echoes described as a list of tuples, as loaded by load_me_data.
 
     w(tCNR) = TE * tSNR
     """
     def weight(echo: nib.Nifti1Image, TE: float):
-        data = echo.get_data()
+        data = echo.get_fdata()
         mean = data[..., -n_vols:].mean(axis=-1)
         std  = data[..., -n_vols:].std(axis=-1)
         return TE * mean / std
@@ -68,37 +71,34 @@ def paid_weights(echoes: List[Tuple[nib.Nifti1Image, float]], n_vols: int) -> np
     return np.stack(weights, axis=-1)
 
 
-def me_combine(pattern: str,
-               outputname: str = '',
-               algorithm: str = 'TE',
-               weights: Optional[List[float]] = None,
-               saveweights: bool = True,
-               volumes: int = 100,
-               logger: str = 'multi-echo') -> int:
-    """General me_combine routine.
+def me_combine(pattern: str, outputname: str='', algorithm: str='TE', weights: Optional[List[float]]=None,
+               saveweights: bool=True, volumes: int=100) -> int:
+    """
+    General me_combine routine.
     Truncates incomplete acquisitions (e.g. when the scanner was stopped manually)
-    Returns an errorcode: 0 = ok, 1 = inconsistent acquisition
 
     Currently supported algorithms:
     - average
     - PAID
     - TE
-    """
 
-    global LOGGER
-    LOGGER = logging.getLogger(logger)
+    Returns an errorcode: 0 = ok, 1 = inconsistent acquisition, 2 = no multi-echo images found
+    """
 
     outputname = Path(outputname)
 
     # Set the logging level and format & add the streamhandler
     if not LOGGER.hasHandlers():
         LOGGER.setLevel(logging.INFO)
-        fmt     = '%(asctime)s - %(name)s - %(levelname)s %(message)s'
+        fmt     = '%(asctime)s - %(levelname)s %(message)s'
         datefmt = '%Y-%m-%d %H:%M:%S'
         coloredlogs.install(level=LOGGER.level, fmt=fmt, datefmt=datefmt)
 
     # Load the data
     me_data, datafiles = load_me_data(Path(pattern), weights)
+    if not datafiles:
+        LOGGER.warning(f"No multi-echo images found in: {pattern}")
+        return 2
 
     # Parse the filenames
     datafile = datafiles[0]
@@ -120,17 +120,18 @@ def me_combine(pattern: str,
             else:
                 LOGGER.error(f"Inconsistent echo images, skipping {pattern} -> {outputname}")
                 return 1
-        echos = np.stack([echo.get_data()[:, :, :, 0:dim4[0]] for echo, TE in me_data], axis=-1)
+        echos = np.stack([echo.get_fdata()[:, :, :, 0:dim4[0]] for echo, TE in me_data], axis=-1)
 
     else:
-        echos = np.stack([echo.get_data() for echo, TE in me_data], axis=-1)
+        echos = np.stack([echo.get_fdata() for echo, TE in me_data], axis=-1)
 
     # Compute the weights
     if algorithm == 'average':
         weights = None
     elif algorithm == 'PAID':
         if me_data[0][0].ndim < 4:
-            LOGGER.error(f"PAID requires 4D data, {datafile} has size: {me_data[0][0].shape()}")
+            LOGGER.error(f"PAID requires 4D data, {datafile} has size: {me_data[0][0].shape}\nSkipping: {pattern} -> {outputname}")
+            return 1
         weights = paid_weights(me_data, volumes)
         # Make the weights have the appropriate number of volumes.
         weights = np.tile(weights[:, :, :, np.newaxis, :], (1, 1, 1, echos.shape[3], 1))
@@ -147,7 +148,7 @@ def me_combine(pattern: str,
     LOGGER.info(f'Saving combined image to: {outputname}')
     if outputname.is_file():
         LOGGER.warning(f'{outputname} already exists, overwriting its content')
-    combined.to_filename(str(outputname))
+    combined.to_filename(outputname)
 
     # Add a combined-echo json sidecar-file
     outputjson = outputname.with_suffix('').with_suffix('.json')
@@ -159,7 +160,7 @@ def me_combine(pattern: str,
             data = json.load(json_fid)
         data['EchoNumber'] = 1
         if algorithm == 'PAID':
-            data['EchoTime'] = np.average([TE for echo, TE in me_data], weights=np.average(np.nan_to_num(weights[..., 0, :]), axis=(0,1,2)))  # This seems to be the best we can do (the BIDS validator indicates there has to be a nr here, an empty value generates a warning)
+            data['EchoTime'] = np.average([TE for echo, TE in me_data], weights=np.nanmean(weights[...,0,:], axis=(0,1,2)))  # This seems to be the best we can do (the BIDS validator indicates there has to be a nr here, an empty value generates a warning)
         else:
             data['EchoTime'] = np.average([TE for echo, TE in me_data], weights=weights)  # This seems to be the best we can do (the BIDS validator indicates there has to be a nr here, an empty value generates a warning)
         with outputjson.open('w') as json_fid:
@@ -168,11 +169,11 @@ def me_combine(pattern: str,
     # Save the weights
     if saveweights and algorithm == 'PAID':
         fname         = (datafile.parent/(datastem + '_combined_weights')).with_suffix(dataext)
-        nifti_weights = nib.Nifti1Image(weights[..., 0, :], combined.affine, combined.header)
+        nifti_weights = nib.Nifti1Image(weights[...,0,:], combined.affine, combined.header)
         LOGGER.info(f'Saving PAID weights to: {fname}')
         if fname.is_file():
             LOGGER.warning(f'{fname} already exists, overwriting its content')
-        nifti_weights.to_filename(str(fname))
+        nifti_weights.to_filename(fname)
 
     return 0
 
@@ -189,18 +190,12 @@ def main():
                                             "  mecombine '/project/number/bids/sub-001/func/*_task-motor_*echo-*.nii.gz'\n"
                                             "  mecombine '/project/number/bids/sub-001/func/*_task-rest_*echo-*.nii.gz' -a PAID\n"
                                             "  mecombine '/project/number/bids/sub-001/func/*_acq-MBME_*run-01*.nii.gz' -w 11 22 33 -o sub-001_task-stroop_acq-mecombined_run-01_bold.nii.gz\n ")
-    parser.add_argument('pattern', type=str,
-                        help='Globlike search pattern with path to select the echo images that need to be combined. Because of the search, be sure to check that not too many files are being read')
-    parser.add_argument('-o','--outputname', type=str, default='',
-                        help="File output name. If not a fullpath name, then the output will be stored in the same folder as the input. If empty, the output filename will be the filename of the first echo appended with a '_combined' suffix")
-    parser.add_argument('-a','--algorithm', default='TE', choices=['PAID', 'TE', 'average'],
-                        help='Combination algorithm. Default: TE')
-    parser.add_argument('-w','--weights', nargs='*', default=None, type=float,
-                        help='Weights (e.g. = echo times) for all echoes')
-    parser.add_argument('-s','--saveweights', action='store_true',
-                        help='If passed and algorithm is PAID, save weights')
-    parser.add_argument('-v','--volumes', type=int, default=100,
-                        help='Number of volumes that is used to compute the weights if algorithm is PAID')
+    parser.add_argument('pattern', type=str, help='Globlike search pattern with path to select the echo images that need to be combined. Because of the search, be sure to check that not too many files are being read')
+    parser.add_argument('-o','--outputname', type=str, default='', help="File output name. If not a fullpath name, then the output will be stored in the same folder as the input. If empty, the output filename will be the filename of the first echo appended with a '_combined' suffix")
+    parser.add_argument('-a','--algorithm', default='TE', choices=['PAID', 'TE', 'average'], help='Combination algorithm. Default: TE')
+    parser.add_argument('-w','--weights', nargs='*', default=None, type=float, help='Weights (e.g. = echo times) for all echoes')
+    parser.add_argument('-s','--saveweights', action='store_true', help='If passed and algorithm is PAID, save weights')
+    parser.add_argument('-v','--volumes', type=int, default=100, help='Number of volumes that is used to compute the weights if algorithm is PAID')
 
     args = parser.parse_args()
 
